%--------------------------------------------------------------------------
%
% Demonstration of Analog Modulations concepts for
% COMMUNICATION THEORY laboratory
%
% Double Sideband Amplitude Modulation
%
% Demostracin de conceptos de Modulaciones Analgicas para 
% el laboratorio de TEORA DE LA COMUNICACIN
%
% Modulacin de amplitud de doble banda lateral
%
%--------------------------------------------------------------------------
%
% LABORATORIO DE TEORIA DE LA COMUNICACION
%
%        Version: 1.0
%  Realizado por: Marcelino Lzaro
%                 Departamento de Teoria de la Seal y Comunicaciones
%                 Universidad Carlos III de Madrid
%      Creacion : marzo 2021
% Actualizacion : marzo 2021
%
%--------------------------------------------------------------------------

% Parmetros temporales
iIni=0;         % Instance inicial
tFin=10;        % Instante final

fc=50;          % Frecuencia de portadora 
faseTx=0;       % Fase de la portadora en transmisin
faseRx=0.0*pi;  % Fase de la portadora en recepcin
fm=5;           % Frecuencia de la seal moduladora sinusoidal
 
fs=1000;        % Frecuencia de muestreo 
Ts=1/fs;        % Tiempo de muestreo

stdRuido=0.2;   % Desviacin estndar de la seal discreta de ruido (gausiano)

nPuntosTexto=20;
nFFT=4*4096;    % Nmero de puntos de la FFT para la representacin frecuencial

% Variable temporal
t=iIni:Ts:tFin;
% Portadora
c=cos(2*pi*fc*t+faseTx);
% Seal moduladora
m=sin(2*pi*fm*t);
% Seal modulada de doble banda lateral (DBL)
sDSB = ammod(m,fc,fs,faseTx);

%--------------------------------------------------------------------------
close all
clc
disp(' ')
disp('--------------------------------------------------------------- ----------------------')
disp(' Demonstrator - Double Sideband Amplitude Modulation (DBL)')
disp('--------------------------------------------------------------- ----------------------')
disp(' ')
disp('Several signals are going to be represented.')
disp('It is recommended to place the figures as they are generated')
disp('in such a way that they can all be seen simultaneously.')
disp(' ')
disp(' Press a key to start...')
pause
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% The carrier signal is plotted
%--------------------------------------------------------------------------
figure(1),plot(t,c)
etiquetas(nPuntosTexto,'Time (t, seconds)','c(t)','Carrier signal')

clc
disp(' ')
disp(['The carrier signal is displayed (frequency ',num2str(fc),' Hz).'])
disp(' ')
disp(' Press a key to highlight the interval between 2 sec and 2.5 sec...')
pause
figure(1),xlim([2,2.5])
pause
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% The modulating signal is plotted
%--------------------------------------------------------------------------
figure(2),plot(t,m)
etiquetas(nPuntosTexto,'Time (t, seg)','m(t)','Modulating signal')
clc
disp(' ')
disp('The modulating signal, which has just been represented, is a ')
disp(['frequency sine ',num2str(fm),' Hz'])
disp(' ')
disp(' Press a key to highlight the interval between 2 sec and 2.5 sec...')
pause
figure(2)
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)
xlim([2,2.5])


%--------------------------------------------------------------------------
% The modulated signal (DSB) is plotted
%--------------------------------------------------------------------------
clc
disp(' ')
disp('The double sideband (DSB) modulated signal is ')
disp('the product between the carrier signal and the modulating signal.')
disp(' ')
disp(' Press a key to see it...')
pause
figure(3),plot(t,c,'r',t,sDSB,'b')
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)

etiquetas(20,'Time (t, sec)','s_{DSB}(t)','Double SideBand (DSB) modulated signal')

clc
disp(' ')
disp(' Press a key to highlight the interval between 2 sec and 2.5 sec...')
pause
figure(3),xlim([2,2.5])
disp(' ')
disp('In red (thin line) the carrier signal has been superimposed, to help')
disp('to understand the characteristics of this modulation')
pause

%--------------------------------------------------------------------------
% FREQUENCY RESPONSES 
%--------------------------------------------------------------------------
clc
disp(' ')
disp('Frequency responses will be represented below.')
disp('The periodogram will be used as an estimate of spectral')
disp('densities.')
disp(' ')
disp(' Press a key to see these responses...')
pause

%--------------------------------------------------------------------------
% Modulating signal
%--------------------------------------------------------------------------
figure(4)
periodogram(m,[],nFFT,fs,'power','centered');
etiquetas(20,'Frequency (Hz)','S_M(jw)','Estimate of the PSD of m(t) (periodogram)')
ylim([-70,0])
xlim([-20,20])
clc
disp(' ')
disp('This is the power spectral density of the signal')
disp('modulating (information signal)')
pause

%--------------------------------------------------------------------------
% Respuesta de la seal modulada
%--------------------------------------------------------------------------
figure(5)
periodogram(sDSB,[],nFFT,fs,'power','centered');
etiquetas(20,'Frequency (Hz)','S_S(jw)','Estimation of the PSD of s(t) (periodogram)')
clc
disp(' ')
disp('This is the power spectral density of the double')
disp('sideband (DSB) modulated signal')
disp(' ')
disp(' Press a key to highlight the band of interest...')
pause
ylim([-70,0])
xlim([30,70])
clc
disp(' ')
disp('Note that only positive frequencies are being represented')
disp('(for negative frequencies the response is symmetrical).')
pause

%------------------------------------------------- -------------------------
% TRANSMISSION - GAUSSIAN ADDITIVE CHANNEL (SIGNAL + GAUSSIAN NOISE)
%------------------------------------------------- -------------------------
clc
disp(' ')
disp(' ')
disp('The signal transmission is simulated below, with')
disp('sum of thermal noise (white and Gaussian noise).')
disp(' ')
disp(' Press a key to continue...')
disp(' ')
pause

%--------------------------------------------------------------------------
% Noise is added during transmission (no attenuation included)
%--------------------------------------------------------------------------
ruido = stdRuido*randn(size(t));

figure(6),plot(t,ruido)
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)

etiquetas(20,'Time (t, sec)','n(t)','Noise signal')
figure(6),xlim([2,2.5])

clc
disp('This is an example of a noise signal (in the time domain)')
disp(' ')
disp('The following is represented in frequency (press a key)...')
pause

%--------------------------------------------------------------------------
% Se muestra la respuesta en frecuencia del ruido
%--------------------------------------------------------------------------
figure(7)
periodogram(ruido,[],nFFT,fs,'power','centered');
etiquetas(20,'Frequency (Hz)','S_n(jw)','Estimation of the PSD of n(t) (periodogram)')
clc
disp(' ')
disp('This is the power spectral density of the noise.')
pause

%------------------------------------------------- -------------------------
% DEMODULATION
%------------------------------------------------- -------------------------
clc
disp(' ')
disp('The sum of modulated signal and noise is demodulated with a ')
disp('coherent receiver (same phase as the transmitter carrier).')
disp(' ')
disp('Firstly it will be demodulated without prior filtering for')
disp('limit noise power (equivalent to assuming filtering with')
disp('a bandwidth of half the sample rate, in this')
disp('case, a bandwidth of 500 Hz')
pause

%------------------------------------------------- -------------------------
% NOISE SIGNAL DEMODULATION - Unfiltered
%------------------------------------------------- -------------------------
% Signal with noise
sDSBn = sDSB + ruido;
% Demodulacin
d=amdemod(sDSBn,fc,fs,faseRx);

figure(8),plot(t,m,'r',t,d,'b')
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)
xlim([2,2.5])

etiquetas(20,'Time (t, sec)','d(t)','Demodulated signal (without filtering)')

indices=find((t>1)&(t<=4));
estimaSNR = cov(m(indices))/cov(d(indices)-m(indices));
estimaSNR_dB = 10*log10(estimaSNR);

disp(' ')
disp('The figure shows the original modulating signal and the demodulated')
disp('signal. The coherent demodulator compensates the scale factor')
disp('half the carrier amplitude.')
disp(' ')
disp(['Estimation of S/N = ',num2str(estimaSNR),' (',num2str(estimaSNR_dB),' dB)'])
disp(' ')
pause

disp('A noise filter is then used prior to demodulation.')
disp('Although theoretically the passband is 45-55 Hz, it is filtered between')
disp('30-70 Hz since the filters are not ideal.')
pause

%------------------------------------------------- -------------------------
% DEMODULATION OF THE NOISE SIGNAL - Filtering before demodulating
%------------------------------------------------- -------------------------
% The signal is filtered before demodulating in the passband
% Although theoretically the pass band is 45-55 Hz,
% is filtered between 30-70 Hz as the filters are not ideal.
sFilt=bandpass(sDSBn,[30,70],fs,'ImpulseResponse','iir','Steepness',0.99);
% Demodulacin
df=amdemod(sFilt,fc,fs,faseRx);

figure(9),plot(t,m,'r',t,df,'b')
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)
xlim([2,2.5])

etiquetas(20,'Time (t, sec)','d(t)','Demodulated signal (with previous filter)')

indices=find((t>1)&(t<=4));
estimaSNRf = cov(m(indices))/cov(df(indices)-m(indices));
estimaSNRf_dB = 10*log10(estimaSNRf);

disp(' ')
disp('The figure shows the original modulating signal and the signal')
disp('demodulated with filtering prior to demodulation.')
disp(' ')
disp(['Estimation of S/N with previous filtering: S/N= ',num2str(estimaSNRf),' (',num2str(estimaSNRf_dB),' dB)'])

