%--------------------------------------------------------------------------
% Demonstration of Analog Modulations concepts for
% COMMUNICATION THEORY laboratory
%
% Demostración de conceptos de Modulaciones Analógicas para 
% el laboratorio de TEORÍA DE LA COMUNICACIÓN
%
% Phase Modulation (PM)
%
%--------------------------------------------------------------------------
%
% LABORATORIO DE TEORÍA DE LA COMUNICACIÓN
%
%        Versión: 1.0
%  Realizado por: Marcelino Lázaro
%                 Departamento de Teoría de la Señal y Comunicaciones
%                 Universidad Carlos III de Madrid
%      Creación : marzo 2023
% Actualización : marzo 2023
%
%--------------------------------------------------------------------------

% Parámetros temporales
iIni=0;         % Instance inicial
tFin=10;        % Instante final

fc=100;         % Frecuencia de portadora 
faseTx=0;       % Fase de la portadora en transmisión
fm=5;           % Frecuencia de la señal moduladora sinusoidal
 
desvFase = pi/2; % Desviación de fase (en rad/s)

fs=2000;        % Frecuencia de muestreo 
Ts=1/fs;        % Tiempo de muestreo

stdRuido=0.2;   % Desviación estándar de la señal discreta de ruido (gausiano)

nPuntosTexto=20;
nFFT=4*4096;    % Número de puntos de la FFT para la representación frecuencial

% Variable temporal
t=iIni:Ts:tFin;
% Portadora
c=cos(2*pi*fc*t+faseTx);
% Señal moduladora
m=sin(2*pi*fm*t);
% Señal modulada en fase (PM)
sPM = pmmod(m,fc,fs,desvFase);

%--------------------------------------------------------------------------
close all
clc
disp(' ')
disp('-------------------------------------------------------------------------------------')
disp(' Demonstrator - Phase Modulation (PM)')
disp('-------------------------------------------------------------------------------------')
disp(' ')
disp('Several signals are going to be represented.')
disp('It is recommended to place the figures as they are generated')
disp('in such a way that they can all be seen simultaneously.')
disp(' ')
disp(' Press a key to start...')
pause
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% Carrier signal is drawn
%--------------------------------------------------------------------------
figure(1),plot(t,c)
etiquetas(nPuntosTexto,'Time (t, seconds)','c(t)','Carrier signal')

clc
disp(' ')
disp(['The carrier signal is displayed (frequency ',num2str(fc),' Hz).'])
disp(' ')
disp(' Press a key to highlight the interval between 2 sec and 2.2 sec...')
pause
figure(1),xlim([2,2.2])
pause
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% The modulating signal
%--------------------------------------------------------------------------
figure(2),plot(t,m)
etiquetas(nPuntosTexto,'Time (t, sec)','m(t)','Modulating signal')
clc
disp(' ')
disp('The modulating signal, which has just been represented, is a ')
disp(['frequency sine ',num2str(fm),' Hz'])
disp(' ')
disp(' Press a key to highlight the interval between 2 sec and 2.2 sec...')
pause
figure(2)
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)
xlim([2,2.2])


%--------------------------------------------------------------------------
% The modulated signal (PM) is represented
%--------------------------------------------------------------------------
clc
disp(' ')
disp('The phase modulated (PM) signal is ')
disp('the product between the carrier signal and the modulating signal.')
disp(' ')
disp(' Press a key to see it...')
pause
figure(3),plot(t,c,'r',t,sPM,'b')
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)

etiquetas(nPuntosTexto,'Time (t, sec)','s_{PM}(t)','PM modulated signal')

clc
disp(' ')
disp(' Press a key to highlight the interval between 2 sec and 2.2 sec...')
pause
figure(3),xlim([2,2.2])
disp(' ')
disp('In red (thin line) the carrier signal has been superimposed, to help')
disp('to understand the characteristics of this modulation')
pause

%------------------------------------------------- -------------------------
% FREQUENCY RESPONSES
%------------------------------------------------- -------------------------
clc
disp(' ')
disp('Frequency responses will be represented below.')
disp('The periodogram will be used as an estimate of densities')
disp('frequency spectra.')
disp(' ')
disp(' Press a key to see these responses...')
pause

%------------------------------------------------- -------------------------
% Modulating signal response
%--------------------------------------------------------------------------
figure(4)
[Sm,faxis]=periodogram(m,[],nFFT,fs,'power','centered');
plot(faxis,10*log10(Sm)), grid
etiquetas(nPuntosTexto,'Frequency (Hz)','S_M(jw)','Estimation of the PSD of m(t) (periodogram)')
ylim([-60,0])
xlim([-10,10])
clc
disp(' ')
disp('This is the power spectral density of the modulating')
disp('signal (information signal)')
pause

%--------------------------------------------------------------------------
% Modulated signal
%--------------------------------------------------------------------------
figure(5)
[SsPM,faxis]=periodogram(sPM,[],nFFT,fs,'power','centered');
plot(faxis,10*log10(SsPM)), grid
etiquetas(nPuntosTexto,'Frequency (Hz)','S_S(jw)','Estimation of the PSD of s(t) (periodogram)')
clc
disp(' ')
disp('This is the power spectral density of the signal')
disp('phase modulated (PM)')
disp(' ')
disp(' Press a key to highlight the band of interest...')
pause
ylim([-60,0])
xlim([70,130])
clc
disp(' ')
disp('Note that only frequencies are being represented')
disp('positive (for negative frequencies the response is symmetrical).')
pause

%--------------------------------------------------------------------------
% TRANSMISSION - GAUSSIAN ADDITIVE CHANNEL (SIGNAL + GAUSSIAN NOISE)
%--------------------------------------------------------------------------
clc
disp(' ')
disp(' ')
disp('The signal transmission is simulated below, with')
disp('sum of thermal noise (white and Gaussian noise).')
disp(' ')
disp(' Press a key to continue...')
disp(' ')
pause

%--------------------------------------------------------------------------
% Noise is added during transmission (no attenuation included)
%--------------------------------------------------------------------------
ruido = stdRuido*randn(size(t));

figure(6),plot(t,ruido)
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)

etiquetas(nPuntosTexto,'Time (t, sec)','n(t)','Noise signal')
figure(6),xlim([2,3])

clc
disp('This is an example of a noise signal (in the time domain)')
disp(' ')
disp('The following is represented in frequency (press a key)...')
pause

%--------------------------------------------------------------------------
% The frequency response of the noise is shown
%--------------------------------------------------------------------------
figure(7)
[Sn,faxis]=periodogram(ruido,[],nFFT,fs,'power','centered');
plot(faxis,10*log10(Sn)), grid
etiquetas(nPuntosTexto,'Frequency (Hz)','S_n(jw)','Estimation of the PSD of n(t) (periodogram)')
clc
disp(' ')
disp('This is the power spectral density of the noise.')
pause

%------------------------------------------------- -------------------------
% DEMODULATION
%------------------------------------------------- -------------------------
clc
disp(' ')
disp('The sum of modulated signal and noise is demodulated with a demodulator')
disp('coherent (same phase as the transmitter carrier).')
disp(' ')
disp('Firstly it will be demodulated without prior filtering for')
disp('limit noise power (equivalent to assuming filtering with')
disp('a bandwidth of half the sample rate, in this')
disp(['case, a bandwidth of ',num2str(fs/2),' Hz'])
pause

%--------------------------------------------------------------------------
% NOISE SIGNAL DEMODULATION - Unfiltered
%--------------------------------------------------------------------------
% Signal with noise
sPMn = sPM + ruido;
% Demodulación
d=pmdemod(sPMn,fc,fs,desvFase);

figure(8),plot(t,m,'r',t,d,'b')
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)
xlim([2,3])

etiquetas(nPuntosTexto,'Time (t, seg)','d(t)','Demodulated signal (without filter)')

indices=find((t>2)&(t<=3));
estimaSNR = cov(d(indices))/cov(d(indices)-m(indices));
estimaSNR_dB = 10*log10(estimaSNR);

disp(' ')
disp('The figure shows the original modulating signal and the signal')
disp('demodulated. The coherent demodulator compensates the scale factor')
disp('half the carrier amplitude.')
disp(' ')
disp(['Estimation of S/N = ',num2str(estimaSNR),' (',num2str(estimaSNR_dB),' dB)'])
disp(' ')
pause

disp('A noise filter is then used prior to demodulation.')
disp('The signal is filtered between 80-120 Hz (the filters are not ideal).')
pause

%--------------------------------------------------------------------------
% DEMODULATION OF THE NOISE SIGNAL - Filtering before demodulating
%--------------------------------------------------------------------------
% The signal is filtered before demodulating in the passband
% Filters between 80-120 Hz (filters are not ideal)
sFilt=bandpass(sPMn,[80,120],fs,'ImpulseResponse','iir','Steepness',0.99);
% Demodulación
df=pmdemod(sFilt,fc,fs,desvFase);

figure(9),plot(t,m,'r',t,df,'b')
ha=gca;
lineas=get(ha,'Children');
set(lineas(1),'LineWidth',3)
xlim([2,3])

etiquetas(nPuntosTexto,'Time (t, sec)','d(t)','Demodulated signal (previously filtered)')

indices=find((t>2)&(t<=3));
estimaSNRf = cov(df(indices))/cov(df(indices)-m(indices));
estimaSNRf_dB = 10*log10(estimaSNRf);

disp(' ')
disp('The figure shows the original modulating signal and the signal')
disp('demodulated with filtering prior to demodulation.')
disp(' ')
disp(['Estimation of S/N with previous filter: S/N= ',num2str(estimaSNRf),' (',num2str(estimaSNRf_dB),' dB)'])

